package com.ghome.sdk.demo.activity;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.DisplayCutout;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;

import com.ghome.gshare.GShareInterface;
import com.ghome.sdk.GHome;
import com.ghome.sdk.common.Constants;
import com.ghome.sdk.common.IGHomeApi;
import com.ghome.sdk.common.IGHomeApi.Callback;
import com.ghome.sdk.demo.R;
import com.ghome.sdk.demo.ui.CheckNullParamFragment;
import com.ghome.sdk.demo.ui.GShareFragment;
import com.ghome.sdk.demo.ui.LoginAreaFragment;
import com.ghome.sdk.demo.ui.MainFragment;
import com.ghome.sdk.demo.ui.PayFragment;
import com.ghome.sdk.demo.ui.StartFragment;
import com.ghome.sdk.demo.util.ConstantsUtil;
import com.ghome.sdk.demo.util.DemoToastUtil;
import com.ghomesdk.gameplus.utils.SharedPreferencesUtil;
import com.ghomesdk.gameplus.utils.log.Log;
import com.ghomesdk.gameplus.utils.network.OkHttpClientInstance;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

public class BaseActivity extends FragmentActivity {
    private static final String TAG = "FragmentActivity";

    public static final int SHOW_FRAGMENT_START = 0;
    public static final int SHOW_FRAGMENT_AREA = 1;
    public static final int SHOW_FRAGMENT_MAIN = 2;
    public static final int SHOW_FRAGMENT_PAY = 3;
    public static final int SHOW_FRAGMENT_CHECK_NULL_PARAM = 4;
    public static final int SHOW_FRAGMENT_SHARE = 5;

    LoginAreaFragment areaFragment;
    StartFragment startFragment;

    MainFragment mainFragment;

    public boolean isInit = false;
    public boolean isLogin = false;
    public boolean isThirdLogin = false;
    public boolean loginEnter = false;

//    private EditText appID = null;

    public String gameId = "";
    public String areaId = "1";

    public boolean isGamePrivacy = false;

    private int resId;
    private Handler mHandler;

    public Callback initCallback = new Callback() {
        @Override
        public void callback(int code, String msg, Map<String, String> data) {
            if (code == 0) {
                //假设是游戏端更新检测到需要强更,弹出强更提示框，玩家点击确认更新的时候调以下接口（游戏非强更场景无需调用） 示例代码：begin
                if (GHome.getInstance().isTapTapChannel()){
                    GHome.getInstance().tapTapUpdateGame(BaseActivity.this, (i, s, map) -> {
                        DemoToastUtil.showToast(BaseActivity.this, "tapTapUpdateGame callback code = " + i + ", msg:" + s);
                        if (i == -1){
                            //取消更新
                            DemoToastUtil.showToast(BaseActivity.this, "玩家取消了强制更新");
                        }
                    });
                }
                //假设是游戏端更新检测到需要强更,弹出强更提示框，玩家点击确认更新的时候调以下接口（游戏非强更场景无需调用） 示例代码：end

                // 初始化成功
                isInit = true;
                SharedPreferencesUtil.setSharedPreferences(BaseActivity.this, ConstantsUtil.APPID, gameId);
                if (loginEnter || isThirdLogin) {
                    GHome.getInstance().login(BaseActivity.this, loginCallback);
                } else {
                    DemoToastUtil.showToast(BaseActivity.this, "beforeLogin success，使用gameId = " + gameId + "#" + areaId);
                }
                loginEnter = false;
            } else if (code == Constants.EVENT_ACCOUNT_LOGOUT) {
                DemoToastUtil.showToast(BaseActivity.this, "注销成功");
                replaceFragment(SHOW_FRAGMENT_START);
                GHome.getInstance().login(BaseActivity.this, loginCallback);
            } else if (code == Constants.EVENT_ACCOUNT_CHANGED) {
                if (data == null){
                    DemoToastUtil.showToast(BaseActivity.this, "data 为空");
                }else{
                    DemoToastUtil.showToast(BaseActivity.this, "切换账号成功 新userid== " + data.get("userid"));
                }
               isLogin = true;
                mHandler.sendEmptyMessage(0);
//
            } else if (code == Constants.EVENT_ACCOUNT_LOGOUT_ONLY) {
                DemoToastUtil.showToast(BaseActivity.this, "注销成功");
                replaceFragment(SHOW_FRAGMENT_START);
            } else if (code == Constants.EVENT_EXIT_GAME) {
                DemoToastUtil.showToast(BaseActivity.this, "退出游戏成功");
                finish();
                android.os.Process.killProcess(android.os.Process.myPid());
                System.exit(0);
            } else {
                DemoToastUtil.showToast(BaseActivity.this, msg + ", 初始失败，使用gameId = " + gameId);
            }
        }
    };

    public Callback loginCallback = new Callback() {
        @Override
        public void callback(int code, String msg, Map<String, String> data) {
            if (code == Constants.ERROR_CODE_SUCCESS) {
                if (data == null) {
                    return;
                }
                SharedPreferencesUtil.setSharedPreferences(BaseActivity.this, ConstantsUtil.USERID, (String) data.get("userid"));
                final String ticket = (String) data.get("ticket");
                BaseActivity.this.runOnUiThread(new Runnable() {

                    @Override
                    public void run() {
                        isLogin = true;
                        mHandler.sendEmptyMessage(0);
                        GHome.getInstance().showFloatIcon(BaseActivity.this, true, 0);
                    }
                });
            } else if (code == Constants.ERROR_CODE_LOGIN_CANCEL) {
                isThirdLogin = false;
                loginEnter = false;
                DemoToastUtil.showToast(BaseActivity.this, msg);
            } else if (code == Constants.ERROR_INIT_NOT_FINISHED) {
                BaseActivity.this.loginEnter = true;
                GHome.getInstance().beforeLogin(BaseActivity.this, BaseActivity.this.gameId, BaseActivity.this.initCallback);
            } else {
                DemoToastUtil.showToast(BaseActivity.this, msg);
            }
        }
    };

    public String showFragmentName() {
        List<Fragment> fragments = this.getSupportFragmentManager().getFragments();
        for (int i = 0; i < fragments.size(); i++) {
            Fragment fragment = fragments.get(i);
            if (fragment != null && fragment.isVisible()) {
                return fragment.getClass().getSimpleName();
            }
        }
        return "";
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        Log.debug(TAG, "onCreate");

        mHandler = new Handler(getMainLooper()) {
            @Override
            public void handleMessage(Message msg) {
                if (msg.what == 0) {
                    replaceFragment(BaseActivity.SHOW_FRAGMENT_AREA);
                }
            }
        };

        final Configuration configuration = this.getResources().getConfiguration(); //获取设置的配置信息
        final int ori = configuration.orientation; //获取屏幕方向

        Log.error("dsafwetsda", "tempOrientation: " + ori);

        if (getWindow() != null && getWindow().getDecorView() != null) {
            getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_STABLE | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN);
            //fullscreen in liuhaiscreen start
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                //设置页面全屏显示
                WindowManager.LayoutParams lp = getWindow().getAttributes();
                lp.layoutInDisplayCutoutMode = WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES;
                //设置页面延伸到刘海区显示
                getWindow().setAttributes(lp);
                //Android P 版本 及以上
                getWindow().getDecorView().post(new Runnable() {
                    @Override
                    public void run() {
                        DisplayCutout displayCutout = getWindow().getDecorView().getRootWindowInsets().getDisplayCutout();
                        if (displayCutout == null) {
                            Log.error("BaseActivity", "没有流海屏");
                        } else {
                            if (ori == Configuration.ORIENTATION_LANDSCAPE) {
                                int notchLeft = displayCutout.getSafeInsetLeft();
                                android.util.Log.e("BaseActivity", "notchLeft : " + notchLeft);
                                getWindow().getDecorView().setPadding(notchLeft, 0, 0, 0);
                            } else {
                                int notchHeight = displayCutout.getSafeInsetTop();
                                android.util.Log.e("BaseActivity", "notchHeight : " + notchHeight);
                                getWindow().getDecorView().setPadding(0, notchHeight, 0, 0);
                            }

                        }
                    }
                });
            }
            //fullscreen in liuhaiscreen end
        }

        super.onCreate(savedInstanceState);

        setContentView(R.layout.ghome_demo_activity_base);
//        appID = findViewById(R.id.editText);
        GHome.getInstance().onCreate(this);

        init();
    }

    private void init() {
        Log.debug(TAG, "init");
//        isThirdLogin = getIntent().getBooleanExtra("thirdLogin", false);
//        if (isThirdLogin) {
//            if (appID != null) {
//                appID.setText("4013");
//                gameId = appID.getText().toString().trim();
//            } else gameId = "4013";
//            GHome.getInstance().initialize(this, gameId, initCallback);
//        }

        resId = R.id.base_fragment;
        setDefaultFragment();


    }

    private void setDefaultFragment() {
        replaceFragment(SHOW_FRAGMENT_START);
    }

    public void replaceFragment(int command) {
        replaceFragment(command, null);
    }

    public void replaceFragment(int command, Bundle bundle) {
        Log.debug(TAG, "replaceFragment command = " + command);
        FragmentManager fm = this.getSupportFragmentManager();
        FragmentTransaction ft = fm.beginTransaction();
        switch (command) {
            case SHOW_FRAGMENT_START:
                startFragment = null;
                startFragment = new StartFragment(BaseActivity.this);
                if (bundle != null) {
                    startFragment.setArguments(bundle);
                }
                ft.replace(resId, startFragment, StartFragment.class.getSimpleName());
                break;
            case SHOW_FRAGMENT_AREA:
                areaFragment = null;
                areaFragment = new LoginAreaFragment(BaseActivity.this);
                ft.replace(resId, areaFragment, LoginAreaFragment.class.getSimpleName());
                ft.addToBackStack(null);
                break;
            case SHOW_FRAGMENT_MAIN:
                mainFragment = null;
                mainFragment = new MainFragment(BaseActivity.this);
                ft.replace(resId, mainFragment, MainFragment.class.getSimpleName());
                if (fm.findFragmentByTag(PayFragment.class.getSimpleName()) == null) {
                    ft.addToBackStack(null);
                }
                break;
            case SHOW_FRAGMENT_PAY:
                PayFragment payFragment = new PayFragment(BaseActivity.this);
                ft.replace(resId, payFragment, PayFragment.class.getSimpleName());
                ft.addToBackStack(null);
                break;
            case SHOW_FRAGMENT_CHECK_NULL_PARAM:
                CheckNullParamFragment checkNullParamFragment = new CheckNullParamFragment(BaseActivity.this);
                ft.replace(resId, checkNullParamFragment, PayFragment.class.getSimpleName());
                ft.addToBackStack(null);
                break;
            case SHOW_FRAGMENT_SHARE:
                GShareFragment gShareFragment = new GShareFragment(BaseActivity.this);
                ft.replace(resId, gShareFragment, GShareFragment.class.getSimpleName());
                ft.addToBackStack(null);
                break;
            default:
                break;
        }
        ft.commitAllowingStateLoss();
    }

    @Override
    protected void onStart() {
        super.onStart();
        GHome.getInstance().onStart(this);
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        GHome.getInstance().onRestart(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        GHome.getInstance().onResume(this);
    }

    @Override
    protected void onPause() {
        GHome.getInstance().onPause(this);
        super.onPause();
    }

    @Override
    protected void onStop() {
        GHome.getInstance().onStop(this);
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        GHome.getInstance().onDestroy(this);

        OkHttpClientInstance.cancelByTag(getClass().getSimpleName());

        //gshare
        GShareInterface.free();
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        GHome.getInstance().onNewIntent(this, intent);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        GHome.getInstance().onActivityResult(this, requestCode, resultCode, data);

        //gshare 20220627
        GShareInterface.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        GHome.getInstance().onRequestPermissionsResult(this, requestCode, permissions, grantResults);

        //gshare 20220627
        GShareInterface.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        GHome.getInstance().onSaveInstanceState(this, outState);
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        GHome.getInstance().onWindowFocusChanged(this, hasFocus);
    }

    public Handler getHandler() {
        return mHandler;
    }

    private long exitTime = 0L;
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (showFragmentName().equals("StartFragment")) {
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                if (System.currentTimeMillis() - exitTime < 2000) {
                    GHome.getInstance().doExtend(this, IGHomeApi.EXTEND_COMMAND_EXIT_PAGE, null, new Callback() {
                        @Override
                        public void callback(int code, String message, Map<String, String> data) {
                            if (code == 0) {
                                GHome.getInstance().doExtend(BaseActivity.this, IGHomeApi.EXTEND_COMMAND_CONFIRM_EXIT_APP, new HashMap<String, String>(), null);
                                finish();
                                System.exit(0);
                            } else {
                                final AlertDialog.Builder builder = new AlertDialog.Builder(BaseActivity.this);
                                builder.setTitle("请确认").setMessage("确定退出游戏吗？").setPositiveButton("确定", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        GHome.getInstance().doExtend(BaseActivity.this, IGHomeApi.EXTEND_COMMAND_CONFIRM_EXIT_APP, new HashMap<String, String>(), null);
                                        finish();
                                        System.exit(0);
                                    }
                                }).setNeutralButton("取消", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                    }
                                });
                                builder.setCancelable(false);
                                builder.show();
                            }
                        }
                    });
                } else {
                    exitTime = System.currentTimeMillis();
                    DemoToastUtil.showToast(this, "如需退出程序请再次点击返回键。");
                }
                return false;
            }
        }


        return super.onKeyDown(keyCode, event);
    }
}
